@@ -1,6 +1,5 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 
-#include <bonobo/bonobo-main.h>
 #include <stdlib.h>
 #include <libecal/e-cal.h>
 
@@ -9,8 +8,7 @@ main (int argc, char **argv)
 {
 	ECal *ecal;
 
-	if (bonobo_init (&argc, argv) == FALSE)
-		g_error ("Could not initialize Bonobo");
+	g_type_init ();
 
 	if (argc < 2) {
 		printf ("usage: test-recur <uid>\n");
