@@ -39,7 +39,7 @@ static void md5_transform (guint32 buf[4], const guint32 in[16]);
 /*
  * Note: this code is not required on little-endian machines.
  */
-static void 
+static void
 _byte_reverse (guchar *buf, guint32 longs)
 {
 	guint32 t;
@@ -54,19 +54,19 @@ _byte_reverse (guchar *buf, guint32 longs)
 
 /**
  * md5_init: Initialise an md5 context object
- * @ctx: md5 context 
- * 
- * Initialise an md5 buffer. 
+ * @ctx: md5 context
+ *
+ * Initialise an md5 buffer.
  *
  **/
-void 
+void
 md5_init (MD5Context *ctx)
 {
 	ctx->buf[0] = 0x67452301;
 	ctx->buf[1] = 0xefcdab89;
 	ctx->buf[2] = 0x98badcfe;
 	ctx->buf[3] = 0x10325476;
-	
+
 	ctx->bits[0] = 0;
 	ctx->bits[1] = 0;
 }
@@ -78,29 +78,29 @@ md5_init (MD5Context *ctx)
  * @ctx: context object used for md5 computaion
  * @buf: buffer to add
  * @len: buffer length
- * 
+ *
  * Update context to reflect the concatenation of another buffer full
  * of bytes. Use this to progressively construct an md5 hash.
  **/
-void 
+void
 md5_update (MD5Context *ctx, const guchar *buf, guint32 len)
 {
 	guint32 t;
-	
+
 	/* Update bitcount */
-	
+
 	t = ctx->bits[0];
 	if ((ctx->bits[0] = t + ((guint32) len << 3)) < t)
 		ctx->bits[1]++;		/* Carry from low to high */
 	ctx->bits[1] += len >> 29;
-	
+
 	t = (t >> 3) & 0x3f;	/* Bytes already in shsInfo->data */
-	
+
 	/* Handle any leading odd-sized chunks */
-	
+
 	if (t) {
 		guchar *p = (guchar *) ctx->in + t;
-		
+
 		t = 64 - t;
 		if (len < t) {
 			memcpy (p, buf, len);
@@ -115,7 +115,7 @@ md5_update (MD5Context *ctx, const guchar *buf, guint32 len)
 		len -= t;
 	}
 	/* Process data in 64-byte chunks */
-	
+
 	while (len >= 64) {
 		memcpy (ctx->in, buf, 64);
 #if G_BYTE_ORDER == G_BIG_ENDIAN
@@ -125,9 +125,9 @@ md5_update (MD5Context *ctx, const guchar *buf, guint32 len)
 		buf += 64;
 		len -= 64;
 	}
-	
+
 	/* Handle any remaining bytes of data. */
-	
+
 	memcpy (ctx->in, buf, len);
 }
 
@@ -136,33 +136,33 @@ md5_update (MD5Context *ctx, const guchar *buf, guint32 len)
 
 
 /*
- * Final wrapup - pad to 64-byte boundary with the bit pattern 
+ * Final wrapup - pad to 64-byte boundary with the bit pattern
  * 1 0* (64-bit count of bits processed, MSB-first)
  */
 /**
  * md5_final: copy the final md5 hash to a bufer
  * @digest: 16 bytes buffer
  * @ctx: context containing the calculated md5
- * 
+ *
  * copy the final md5 hash to a bufer
  **/
-void 
+void
 md5_final (MD5Context *ctx, guchar digest[16])
 {
 	guint32 count;
 	guchar *p;
-	
+
 	/* Compute number of bytes mod 64 */
 	count = (ctx->bits[0] >> 3) & 0x3F;
-	
+
 	/* Set the first char of padding to 0x80.  This is safe since there is
 	   always at least one byte free */
 	p = ctx->in + count;
 	*p++ = 0x80;
-	
+
 	/* Bytes of padding needed to make 64 bytes */
 	count = 64 - 1 - count;
-	
+
 	/* Pad out to 56 mod 64 */
 	if (count < 8) {
 		/* Two lots of padding:  Pad the first block to 64 bytes */
@@ -171,7 +171,7 @@ md5_final (MD5Context *ctx, guchar digest[16])
 		_byte_reverse (ctx->in, 16);
 #endif
 		md5_transform (ctx->buf, (guint32 *) ctx->in);
-		
+
 		/* Now fill the next block with 56 bytes */
 		memset (ctx->in, 0, 56);
 	} else {
@@ -181,11 +181,11 @@ md5_final (MD5Context *ctx, guchar digest[16])
 #if G_BYTE_ORDER == G_BIG_ENDIAN
 	_byte_reverse (ctx->in, 14);
 #endif
-	
+
 	/* Append length in bits and transform */
 	((guint32 *) ctx->in)[14] = ctx->bits[0];
 	((guint32 *) ctx->in)[15] = ctx->bits[1];
-	
+
 	md5_transform (ctx->buf, (guint32 *) ctx->in);
 #if G_BYTE_ORDER == G_BIG_ENDIAN
 	_byte_reverse ((guchar *) ctx->buf, 4);
@@ -213,16 +213,16 @@ md5_final (MD5Context *ctx, guchar digest[16])
  * reflect the addition of 16 longwords of new data.  md5_Update blocks
  * the data and converts bytes into longwords for this routine.
  */
-static void 
+static void
 md5_transform (guint32 buf[4], const guint32 in[16])
 {
 	register guint32 a, b, c, d;
-	
+
 	a = buf[0];
 	b = buf[1];
 	c = buf[2];
 	d = buf[3];
-	
+
 	MD5STEP (F1, a, b, c, d, in[0] + 0xd76aa478, 7);
 	MD5STEP (F1, d, a, b, c, in[1] + 0xe8c7b756, 12);
 	MD5STEP (F1, c, d, a, b, in[2] + 0x242070db, 17);
@@ -239,7 +239,7 @@ md5_transform (guint32 buf[4], const guint32 in[16])
 	MD5STEP (F1, d, a, b, c, in[13] + 0xfd987193, 12);
 	MD5STEP (F1, c, d, a, b, in[14] + 0xa679438e, 17);
 	MD5STEP (F1, b, c, d, a, in[15] + 0x49b40821, 22);
-	
+
 	MD5STEP (F2, a, b, c, d, in[1] + 0xf61e2562, 5);
 	MD5STEP (F2, d, a, b, c, in[6] + 0xc040b340, 9);
 	MD5STEP (F2, c, d, a, b, in[11] + 0x265e5a51, 14);
@@ -256,7 +256,7 @@ md5_transform (guint32 buf[4], const guint32 in[16])
 	MD5STEP (F2, d, a, b, c, in[2] + 0xfcefa3f8, 9);
 	MD5STEP (F2, c, d, a, b, in[7] + 0x676f02d9, 14);
 	MD5STEP (F2, b, c, d, a, in[12] + 0x8d2a4c8a, 20);
-	
+
 	MD5STEP (F3, a, b, c, d, in[5] + 0xfffa3942, 4);
 	MD5STEP (F3, d, a, b, c, in[8] + 0x8771f681, 11);
 	MD5STEP (F3, c, d, a, b, in[11] + 0x6d9d6122, 16);
@@ -273,7 +273,7 @@ md5_transform (guint32 buf[4], const guint32 in[16])
 	MD5STEP (F3, d, a, b, c, in[12] + 0xe6db99e5, 11);
 	MD5STEP (F3, c, d, a, b, in[15] + 0x1fa27cf8, 16);
 	MD5STEP (F3, b, c, d, a, in[2] + 0xc4ac5665, 23);
-	
+
 	MD5STEP (F4, a, b, c, d, in[0] + 0xf4292244, 6);
 	MD5STEP (F4, d, a, b, c, in[7] + 0x432aff97, 10);
 	MD5STEP (F4, c, d, a, b, in[14] + 0xab9423a7, 15);
@@ -290,7 +290,7 @@ md5_transform (guint32 buf[4], const guint32 in[16])
 	MD5STEP (F4, d, a, b, c, in[11] + 0xbd3af235, 10);
 	MD5STEP (F4, c, d, a, b, in[2] + 0x2ad7d2bb, 15);
 	MD5STEP (F4, b, c, d, a, in[9] + 0xeb86d391, 21);
-	
+
 	buf[0] += a;
 	buf[1] += b;
 	buf[2] += c;
@@ -305,19 +305,29 @@ md5_transform (guint32 buf[4], const guint32 in[16])
  * @buffer: byte buffer
  * @buffer_size: buffer size (in bytes)
  * @digest: 16 bytes buffer receiving the hash code.
- * 
- * Get the md5 hash of a buffer. The result is put in 
+ *
+ * Get the md5 hash of a buffer. The result is put in
  * the 16 bytes buffer @digest .
  **/
 void
 md5_get_digest (const gchar *buffer, gint buffer_size, guchar digest[16])
-{	
-	MD5Context ctx;
+{
+	GChecksum *checksum;
+	guint8 *local_digest;
+	gsize length;
 
-	md5_init (&ctx);
-	md5_update (&ctx, (guchar*)buffer, buffer_size);
-	md5_final (&ctx, digest);
-	
+	g_return_if_fail (buffer_size > 1);
+	g_return_if_fail (digest != NULL);
+
+	length = g_checksum_type_get_length (G_CHECKSUM_MD5);
+	local_digest = g_alloca (length);
+
+	checksum = g_checksum_new (G_CHECKSUM_MD5);
+	g_checksum_update (checksum, (const guchar *) buffer, buffer_size);
+	g_checksum_get_digest (checksum, local_digest, &length);
+	g_checksum_free (checksum);
+
+	memcpy (digest, local_digest, length);
 }
 
 
@@ -325,32 +335,32 @@ md5_get_digest (const gchar *buffer, gint buffer_size, guchar digest[16])
  * md5_get_digest_from_file: get the md5 hash of a file
  * @filename: file name
  * @digest: 16 bytes buffer receiving the hash code.
- * 
- * Get the md5 hash of a file. The result is put in 
+ *
+ * Get the md5 hash of a file. The result is put in
  * the 16 bytes buffer @digest .
  **/
 void
 md5_get_digest_from_file (const gchar *filename, guchar digest[16])
-{	
-	MD5Context ctx;
-	guchar tmp_buf[1024];
-	gint nb_bytes_read;
-	FILE *fp;
+{
+	GMappedFile *mapped_file;
+	const gchar *contents;
+	gsize length;
+	GError *error = NULL;
 
-	md5_init (&ctx);
-	fp = g_fopen(filename, "rb");
-	if (!fp) {
+	g_return_if_fail (filename != NULL);
+	g_return_if_fail (digest != NULL);
+
+	mapped_file = g_mapped_file_new (filename, FALSE, &error);
+	if (mapped_file == NULL) {
+		/* XXX No way to report errors with this API. */
+		g_warning ("%s", error->message);
 		return;
 	}
-	
-	while ((nb_bytes_read = fread (tmp_buf, 1, sizeof (tmp_buf), fp)) > 0)
-		md5_update (&ctx, tmp_buf, nb_bytes_read);
-	
-	if (ferror(fp)) {
-		fclose(fp);
-		return;
-	}
-	fclose(fp);
-	
-	md5_final (&ctx, digest);
+
+	contents = g_mapped_file_get_contents (mapped_file);
+	length = g_mapped_file_get_length (mapped_file);
+
+	md5_get_digest (contents, length, digest);
+
+	g_mapped_file_free (mapped_file);
 }
